@@ -1,5 +1,5 @@
 package WebAPI::DBIC::Resource::GenericSet;
-$WebAPI::DBIC::Resource::GenericSet::VERSION = '0.001009';
+$WebAPI::DBIC::Resource::GenericSet::VERSION = '0.002000';
 
 use Moo;
 use namespace::clean;
@@ -11,6 +11,10 @@ with    'WebAPI::DBIC::Resource::Role::Set',
         'WebAPI::DBIC::Resource::Role::DBIC_HAL', # XXX move out?
         'WebAPI::DBIC::Resource::Role::SetHAL',
         'WebAPI::DBIC::Resource::Role::SetWritableHAL',
+        # Enable JSON API support:
+        'WebAPI::DBIC::Resource::Role::DBIC_JSONAPI', # XXX move out?
+        'WebAPI::DBIC::Resource::Role::SetJSONAPI',
+        'WebAPI::DBIC::Resource::Role::SetWritableJSONAPI',
         ;
 
 1;
@@ -27,7 +31,7 @@ WebAPI::DBIC::Resource::GenericSet
 
 =head1 VERSION
 
-version 0.001009
+version 0.002000
 
 =head1 NAME
 
