@@ -1,5 +1,5 @@
 package WebAPI::DBIC::Resource::Role::DBIC;
-$WebAPI::DBIC::Resource::Role::DBIC::VERSION = '0.001009';
+$WebAPI::DBIC::Resource::Role::DBIC::VERSION = '0.002000';
 
 use Carp qw(croak confess);
 use Devel::Dwarn;
@@ -27,7 +27,7 @@ has writable => (
 
 has prefetch => (
     is => 'rw',
-    default => sub { {} },
+    default => sub { [] },
 );
 
 
@@ -73,7 +73,7 @@ sub web_machine_resource {
         request  => $self->request,
         response => $self->request->new_response,
         throwable => $self->throwable,
-        prefetch  => {}, # don't propagate prefetch by default
+        prefetch  => [], # don't propagate prefetch by default
         set => undef,
         id => undef,
         #  XXX others? which and why? generalize
@@ -154,7 +154,7 @@ WebAPI::DBIC::Resource::Role::DBIC
 
 =head1 VERSION
 
-version 0.001009
+version 0.002000
 
 =head1 NAME
 
